
integer zz_log_handle = -1

integer proc zz_log_write(string msg)
  integer old_DateFormat = Set(DateFormat, 6)
  integer old_timeFormat = Set(TimeFormat, 1)
  integer ok             = TRUE

  #ifdef LINUX
    string NEWLINE [1] = Chr(10)
  #else
    string NEWLINE [2] = Chr(13) + Chr(10)
  #endif

  if zz_log_handle == -1
    Warn('No open log to write to.')
    ok = FALSE
  else
    if fWrite(zz_log_handle, GetDateStr() + ' ' + GetTimeStr() + ' ') == -1
      Warn('Log write date time prefix failed.')
      ok = FALSE
    else
      if fWrite(zz_log_handle, msg) == -1
        Warn('Log write message failed.')
        ok = FALSE
      else
        if fWrite(zz_log_handle, NEWLINE) == -1
          Warn('Log write newline failed.')
          ok = FALSE
        endif
      endif
    endif
  endif
  Set(DateFormat, old_DateFormat)
  Set(TimeFormat, old_timeFormat)
  return(ok)
end zz_log_write

integer proc zz_log_open()
  string  log_fqn [MAXSTRINGLEN] = 'tse_log_' +
                                   SplitPath(CurrMacroFilename(), _NAME_) +
                                   '_' + Format(GetTime():7:'0') + '.log'
  integer log_attrs              = 0
  integer ok                     = TRUE

  #ifdef LINUX
    log_fqn = '/tmp/' + log_fqn
  #else
    log_fqn = GetEnvStr('TMP') + '\' + log_fqn
  #endif

  if zz_log_handle <> -1
    Warn('Log already open.')
  else
    log_attrs = FileExists(log_fqn)
    if log_attrs
      if log_attrs & _DIRECTORY_
        Warn('Cannot log to directory.')
        ok = FALSE
      else
        zz_log_handle = fOpen(log_fqn, _OPEN_READWRITE_)
        if zz_log_handle == -1
          Warn('Open log failed.')
          ok = FALSE
        else
          if fSeek(zz_log_handle, 0, _SEEK_END_) == -1
            Warn('First log fSeek failed.')
            ok = FALSE
          else
            zz_log_write('Log reopened.')
          endif
        endif
      endif
    else
      if FileExists(SplitPath(log_fqn, _DRIVE_|_PATH_)) & _DIRECTORY_
        zz_log_handle = fCreate(log_fqn)
        if zz_log_handle == -1
          Warn('Creating log failed.')
          ok = FALSE
        else
          ok = zz_log_write('Log opened.')
        endif
      else
        Warn('Cannot log to non-existing directory.')
        ok = FALSE
      endif
    endif
  endif

  return(ok)
end zz_log_open

integer proc zz_log_close()
  integer ok = FALSE
  if zz_log_handle <> -1
    ok = zz_log_write('Log closed.')
    if fClose(zz_log_handle)
      zz_log_handle = -1
    else
      Warn('Failed to close log.')
      ok = FALSE
    endif
  endif
  return(ok)
end zz_log_close

